/*************************************************************************
 *
 *  C-to-C Translator
 *
 *  Adapted from Clean ANSI C Parser
 *  Eric A. Brewer, Michael D. Noakes
 * 
 *************************************************************************/

#include "basics.h"

#ifndef NO_STDARG
#include <stdarg.h>
#define VA_START(ap,fmt)   va_start(ap,fmt)
#else
   /* for many older Unix platforms: use varargs */
#include <varargs.h>
extern int vfprintf(FILE *, const char *, void *);
#define VA_START(ap,fmt)   va_start(ap)
#endif

GLOBAL int Line = 1, Errors = 0, Warnings = 0;
GLOBAL int LineOffset = 0;

GLOBAL NoReturn Fail(const char *file, int line, const char *msg)
{
     fprintf(stderr, "Assertion failed in %s, line %d\n", file, line);
     fprintf(stderr, "\t%s\n", msg);

     if (strcmp(PhaseName, "Parsing") == 0) {
	  fprintf(stderr, "Input: %s, line %d\n", Filename, Line);
     }
#if 0
     exit(10);
#endif
     abort();
}

GLOBAL void SyntaxError(const char *fmt,...)
{
     va_list ap;
     VA_START(ap, fmt);

     Errors++;
     fprintf(stderr, "%s:%d: ", Filename, Line);
     vfprintf(stderr, fmt, ap);
     fputc('\n', stderr);
     va_end(ap);
}

GLOBAL void Warning(int level, const char *fmt,...)
{
     va_list ap;
     VA_START(ap, fmt);

     if (level > WarningLevel)
	  return;
     Warnings++;
     fprintf(stderr, "%s:%d: warning: ", Filename, Line);
     vfprintf(stderr, fmt, ap);
     fputc('\n', stderr);
     va_end(ap);
}

GLOBAL void Hint(const char *fmt,...)
{
     va_list ap;
     VA_START(ap, fmt);

     fprintf(stderr, "%s:%d: hint: ", Filename, Line);
     vfprintf(stderr, fmt, ap);
     fputc('\n', stderr);
     va_end(ap);
}

GLOBAL void SyntaxErrorCoord(Coord c, const char *fmt,...)
{
     va_list ap;
     VA_START(ap, fmt);

     Errors++;
     PRINT_COORD(stderr, c);
     fprintf(stderr, ": ");
     vfprintf(stderr, fmt, ap);
     fputc('\n', stderr);
     va_end(ap);
}

GLOBAL void WarningCoord(int level, Coord c, const char *fmt,...)
{
     va_list ap;
     VA_START(ap, fmt);

     if (level > WarningLevel)
	  return;
     Warnings++;
     PRINT_COORD(stderr, c);
     fprintf(stderr, ": warning: ");
     vfprintf(stderr, fmt, ap);
     fputc('\n', stderr);
     va_end(ap);
}

GLOBAL void HintCoord(Coord c, const char *fmt,...)
{
     va_list ap;
     VA_START(ap, fmt);

     PRINT_COORD(stderr, c);
     fprintf(stderr, ": hint: ");
     vfprintf(stderr, fmt, ap);
     fputc('\n', stderr);
     va_end(ap);
}
